from .exec import exec_flumina_script, _ExecMode
from .logging import get_logger
from .util import log_time

import argparse
import os
import json


def export_command(args):
    file_path = args.path

    # First try to run the script under 'meta' device
    with log_time(f"Running script {file_path} under meta device context"):
        try:
            exec_flumina_script(file_path, exec_mode=_ExecMode.SERVE, device="meta")
        except Exception as e:
            get_logger().exception(
                f"Flumina script failed while running under meta device context. It is "
                f"highly recommended that Flumina scripts be made to "
                f"run where all parameters are initialized as meta device. "
                f"This ensures fast load times and efficiency when deployed "
                f"to Fireworks. Please check your script and try again"
            )

    with log_time(f"Running script {file_path} for export"):
        exported_mod, code = exec_flumina_script(file_path)

        dir = args.output_path
        if not os.path.exists(dir):
            os.mkdir(dir)

        with open(os.path.join(dir, "model.safetensors"), "wb") as f, log_time(
            "Writing exported weights to disk"
        ):
            f.write(exported_mod.serialized_weights)

        with open(os.path.join(dir, "flumina.py"), "w") as f, log_time(
            "Writing exported code to disk"
        ):
            f.write(code)

        with open(os.path.join(dir, "fireworks.json"), "w") as f, log_time(
            "Writing model metadata to disk"
        ):
            json.dump(
                {"_is_flumina_model": True, "paths": exported_mod.path_to_method_name},
                f,
            )


def main():
    # Create the top-level parser
    parser = argparse.ArgumentParser(description="Flumina CLI Tool")
    subparsers = parser.add_subparsers(
        dest="command",
        required=True,
        help="The Flumina command to run",
    )

    # Create the parser for the "run" command
    parser_run = subparsers.add_parser(
        "export", help="Run a Flumina script and upload contained Models"
    )
    parser_run.add_argument("path", type=str, help="Path to the Flumina script to run")
    parser_run.add_argument(
        "--output-path",
        type=str,
        required=False,
        help="Output directory to which to serialize the model.",
    )
    parser_run.set_defaults(func=export_command)

    # Parse the arguments
    args = parser.parse_args()

    # Dispatch to the appropriate function based on the command
    if hasattr(args, "func"):
        args.func(args)
    else:
        parser.print_help()


if __name__ == "__main__":
    main()



from .api_client import FireworksClient
from .api import ListModelsResponse


class Model:
    @classmethod
    def list(cls, request_timeout=60):
        """Returns a list of available models.

        Args:
          request_timeout (int, optional): The request timeout in seconds. Default is 60.

        Returns:
          ListModelsResponse: A list of available models.
        """
        client = FireworksClient(request_timeout=request_timeout)
        response = client._get_request(f"{client.base_url}/models")
        return ListModelsResponse(**response)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gateway/gateway.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import account_pb2 as gateway_dot_account__pb2
from . import aws_iam_role_binding_pb2 as gateway_dot_aws__iam__role__binding__pb2
from . import batch_job_pb2 as gateway_dot_batch__job__pb2
from . import cluster_pb2 as gateway_dot_cluster__pb2
from . import deployment_pb2 as gateway_dot_deployment__pb2
from . import environment_pb2 as gateway_dot_environment__pb2
from . import model_pb2 as gateway_dot_model__pb2
from . import node_pool_pb2 as gateway_dot_node__pool__pb2
from . import sign_in_pb2 as gateway_dot_sign__in__pb2
from . import snapshot_pb2 as gateway_dot_snapshot__pb2
from . import user_pb2 as gateway_dot_user__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='gateway/gateway.proto',
  package='gateway',
  syntax='proto3',
  serialized_options=b'Z\n./;gateway',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x15gateway/gateway.proto\x12\x07gateway\x1a\x15gateway/account.proto\x1a\"gateway/aws_iam_role_binding.proto\x1a\x17gateway/batch_job.proto\x1a\x15gateway/cluster.proto\x1a\x18gateway/deployment.proto\x1a\x19gateway/environment.proto\x1a\x13gateway/model.proto\x1a\x17gateway/node_pool.proto\x1a\x15gateway/sign_in.proto\x1a\x16gateway/snapshot.proto\x1a\x12gateway/user.proto\x1a\x1bgoogle/protobuf/empty.proto2\xb7&\n\x07Gateway\x12:\n\x06SignIn\x12\x16.gateway.SignInRequest\x1a\x16.google.protobuf.Empty\"\x00\x12\\\n\x11GetOAuthArguments\x12!.gateway.GetOAuthArgumentsRequest\x1a\".gateway.GetOAuthArgumentsResponse\"\x00\x12\x42\n\rCreateAccount\x12\x1d.gateway.CreateAccountRequest\x1a\x10.gateway.Account\"\x00\x12<\n\nGetAccount\x12\x1a.gateway.GetAccountRequest\x1a\x10.gateway.Account\"\x00\x12M\n\x0cListAccounts\x12\x1c.gateway.ListAccountsRequest\x1a\x1d.gateway.ListAccountsResponse\"\x00\x12\x42\n\rUpdateAccount\x12\x1d.gateway.UpdateAccountRequest\x1a\x10.gateway.Account\"\x00\x12H\n\rDeleteAccount\x12\x1d.gateway.DeleteAccountRequest\x1a\x16.google.protobuf.Empty\"\x00\x12\x45\n\x0e\x43reateBatchJob\x12\x1e.gateway.CreateBatchJobRequest\x1a\x11.gateway.BatchJob\"\x00\x12?\n\x0bGetBatchJob\x12\x1b.gateway.GetBatchJobRequest\x1a\x11.gateway.BatchJob\"\x00\x12P\n\rListBatchJobs\x12\x1d.gateway.ListBatchJobsRequest\x1a\x1e.gateway.ListBatchJobsResponse\"\x00\x12J\n\x0e\x44\x65leteBatchJob\x12\x1e.gateway.DeleteBatchJobRequest\x1a\x16.google.protobuf.Empty\"\x00\x12V\n\x14\x42\x61tchDeleteBatchJobs\x12$.gateway.BatchDeleteBatchJobsRequest\x1a\x16.google.protobuf.Empty\"\x00\x12J\n\x0e\x43\x61ncelBatchJob\x12\x1e.gateway.CancelBatchJobRequest\x1a\x16.google.protobuf.Empty\"\x00\x12\x42\n\rCreateCluster\x12\x1d.gateway.CreateClusterRequest\x1a\x10.gateway.Cluster\"\x00\x12<\n\nGetCluster\x12\x1a.gateway.GetClusterRequest\x1a\x10.gateway.Cluster\"\x00\x12M\n\x0cListClusters\x12\x1c.gateway.ListClustersRequest\x1a\x1d.gateway.ListClustersResponse\"\x00\x12\x42\n\rUpdateCluster\x12\x1d.gateway.UpdateClusterRequest\x1a\x10.gateway.Cluster\"\x00\x12H\n\rDeleteCluster\x12\x1d.gateway.DeleteClusterRequest\x1a\x16.google.protobuf.Empty\"\x00\x12\x66\n\x18GetClusterConnectionInfo\x12(.gateway.GetClusterConnectionInfoRequest\x1a\x1e.gateway.ClusterConnectionInfo\"\x00\x12N\n\x11\x43reateEnvironment\x12!.gateway.CreateEnvironmentRequest\x1a\x14.gateway.Environment\"\x00\x12H\n\x0eGetEnvironment\x12\x1e.gateway.GetEnvironmentRequest\x1a\x14.gateway.Environment\"\x00\x12Y\n\x10ListEnvironments\x12 .gateway.ListEnvironmentsRequest\x1a!.gateway.ListEnvironmentsResponse\"\x00\x12N\n\x11UpdateEnvironment\x12!.gateway.UpdateEnvironmentRequest\x1a\x14.gateway.Environment\"\x00\x12P\n\x11\x44\x65leteEnvironment\x12!.gateway.DeleteEnvironmentRequest\x1a\x16.google.protobuf.Empty\"\x00\x12\\\n\x17\x42\x61tchDeleteEnvironments\x12\'.gateway.BatchDeleteEnvironmentsRequest\x1a\x16.google.protobuf.Empty\"\x00\x12R\n\x12\x43onnectEnvironment\x12\".gateway.ConnectEnvironmentRequest\x1a\x16.google.protobuf.Empty\"\x00\x12X\n\x15\x44isconnectEnvironment\x12%.gateway.DisconnectEnvironmentRequest\x1a\x16.google.protobuf.Empty\"\x00\x12\x45\n\x0e\x43reateSnapshot\x12\x1e.gateway.CreateSnapshotRequest\x1a\x11.gateway.Snapshot\"\x00\x12?\n\x0bGetSnapshot\x12\x1b.gateway.GetSnapshotRequest\x1a\x11.gateway.Snapshot\"\x00\x12P\n\rListSnapshots\x12\x1d.gateway.ListSnapshotsRequest\x1a\x1e.gateway.ListSnapshotsResponse\"\x00\x12J\n\x0e\x44\x65leteSnapshot\x12\x1e.gateway.DeleteSnapshotRequest\x1a\x16.google.protobuf.Empty\"\x00\x12V\n\x0fGetBatchJobLogs\x12\x1f.gateway.GetBatchJobLogsRequest\x1a .gateway.GetBatchJobLogsResponse\"\x00\x12\x45\n\x0e\x43reateNodePool\x12\x1e.gateway.CreateNodePoolRequest\x1a\x11.gateway.NodePool\"\x00\x12?\n\x0bGetNodePool\x12\x1b.gateway.GetNodePoolRequest\x1a\x11.gateway.NodePool\"\x00\x12P\n\rListNodePools\x12\x1d.gateway.ListNodePoolsRequest\x1a\x1e.gateway.ListNodePoolsResponse\"\x00\x12\x45\n\x0eUpdateNodePool\x12\x1e.gateway.UpdateNodePoolRequest\x1a\x11.gateway.NodePool\"\x00\x12J\n\x0e\x44\x65leteNodePool\x12\x1e.gateway.DeleteNodePoolRequest\x1a\x16.google.protobuf.Empty\"\x00\x12V\n\x14\x42\x61tchDeleteNodePools\x12$.gateway.BatchDeleteNodePoolsRequest\x1a\x16.google.protobuf.Empty\"\x00\x12N\n\x10GetNodePoolStats\x12 .gateway.GetNodePoolStatsRequest\x1a\x16.gateway.NodePoolStats\"\x00\x12\x39\n\nCreateUser\x12\x1a.gateway.CreateUserRequest\x1a\r.gateway.User\"\x00\x12\x33\n\x07GetUser\x12\x17.gateway.GetUserRequest\x1a\r.gateway.User\"\x00\x12\x44\n\tListUsers\x12\x19.gateway.ListUsersRequest\x1a\x1a.gateway.ListUsersResponse\"\x00\x12\x39\n\nUpdateUser\x12\x1a.gateway.UpdateUserRequest\x1a\r.gateway.User\"\x00\x12\x42\n\nDeleteUser\x12\x1a.gateway.DeleteUserRequest\x1a\x16.google.protobuf.Empty\"\x00\x12<\n\x0b\x43reateModel\x12\x1b.gateway.CreateModelRequest\x1a\x0e.gateway.Model\"\x00\x12\x36\n\x08GetModel\x12\x18.gateway.GetModelRequest\x1a\x0e.gateway.Model\"\x00\x12G\n\nListModels\x12\x1a.gateway.ListModelsRequest\x1a\x1b.gateway.ListModelsResponse\"\x00\x12<\n\x0bUpdateModel\x12\x1b.gateway.UpdateModelRequest\x1a\x0e.gateway.Model\"\x00\x12\x44\n\x0b\x44\x65leteModel\x12\x1b.gateway.DeleteModelRequest\x1a\x16.google.protobuf.Empty\"\x00\x12k\n\x16GetModelUploadEndpoint\x12&.gateway.GetModelUploadEndpointRequest\x1a\'.gateway.GetModelUploadEndpointResponse\"\x00\x12T\n\x13ValidateModelUpload\x12#.gateway.ValidateModelUploadRequest\x1a\x16.google.protobuf.Empty\"\x00\x12\x44\n\x0b\x44\x65ployModel\x12\x1b.gateway.DeployModelRequest\x1a\x16.google.protobuf.Empty\"\x00\x12H\n\rUndeployModel\x12\x1d.gateway.UndeployModelRequest\x1a\x16.google.protobuf.Empty\"\x00\x12K\n\x10\x43reateDeployment\x12 .gateway.CreateDeploymentRequest\x1a\x13.gateway.Deployment\"\x00\x12\x45\n\rGetDeployment\x12\x1d.gateway.GetDeploymentRequest\x1a\x13.gateway.Deployment\"\x00\x12I\n\x0fStartDeployment\x12\x1f.gateway.StartDeploymentRequest\x1a\x13.gateway.Deployment\"\x00\x12V\n\x0fListDeployments\x12\x1f.gateway.ListDeploymentsRequest\x1a .gateway.ListDeploymentsResponse\"\x00\x12K\n\x10UpdateDeployment\x12 .gateway.UpdateDeploymentRequest\x1a\x13.gateway.Deployment\"\x00\x12N\n\x10\x44\x65leteDeployment\x12 .gateway.DeleteDeploymentRequest\x1a\x16.google.protobuf.Empty\"\x00\x12`\n\x17\x43reateAwsIamRoleBinding\x12\'.gateway.CreateAwsIamRoleBindingRequest\x1a\x1a.gateway.AwsIamRoleBinding\"\x00\x12k\n\x16ListAwsIamRoleBindings\x12&.gateway.ListAwsIamRoleBindingsRequest\x1a\'.gateway.ListAwsIamRoleBindingsResponse\"\x00\x12\\\n\x17\x44\x65leteAwsIamRoleBinding\x12\'.gateway.DeleteAwsIamRoleBindingRequest\x1a\x16.google.protobuf.Empty\"\x00\x12N\n\x10ReconcileCluster\x12 .gateway.ReconcileClusterRequest\x1a\x16.google.protobuf.Empty\"\x00\x12P\n\x11ReconcileNodePool\x12!.gateway.ReconcileNodePoolRequest\x1a\x16.google.protobuf.Empty\"\x00\x42\x0cZ\n./;gatewayb\x06proto3'
  ,
  dependencies=[gateway_dot_account__pb2.DESCRIPTOR,gateway_dot_aws__iam__role__binding__pb2.DESCRIPTOR,gateway_dot_batch__job__pb2.DESCRIPTOR,gateway_dot_cluster__pb2.DESCRIPTOR,gateway_dot_deployment__pb2.DESCRIPTOR,gateway_dot_environment__pb2.DESCRIPTOR,gateway_dot_model__pb2.DESCRIPTOR,gateway_dot_node__pool__pb2.DESCRIPTOR,gateway_dot_sign__in__pb2.DESCRIPTOR,gateway_dot_snapshot__pb2.DESCRIPTOR,gateway_dot_user__pb2.DESCRIPTOR,google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,])



_sym_db.RegisterFileDescriptor(DESCRIPTOR)


DESCRIPTOR._options = None

_GATEWAY = _descriptor.ServiceDescriptor(
  name='Gateway',
  full_name='gateway.Gateway',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=337,
  serialized_end=5256,
  methods=[
  _descriptor.MethodDescriptor(
    name='SignIn',
    full_name='gateway.Gateway.SignIn',
    index=0,
    containing_service=None,
    input_type=gateway_dot_sign__in__pb2._SIGNINREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetOAuthArguments',
    full_name='gateway.Gateway.GetOAuthArguments',
    index=1,
    containing_service=None,
    input_type=gateway_dot_sign__in__pb2._GETOAUTHARGUMENTSREQUEST,
    output_type=gateway_dot_sign__in__pb2._GETOAUTHARGUMENTSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CreateAccount',
    full_name='gateway.Gateway.CreateAccount',
    index=2,
    containing_service=None,
    input_type=gateway_dot_account__pb2._CREATEACCOUNTREQUEST,
    output_type=gateway_dot_account__pb2._ACCOUNT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetAccount',
    full_name='gateway.Gateway.GetAccount',
    index=3,
    containing_service=None,
    input_type=gateway_dot_account__pb2._GETACCOUNTREQUEST,
    output_type=gateway_dot_account__pb2._ACCOUNT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListAccounts',
    full_name='gateway.Gateway.ListAccounts',
    index=4,
    containing_service=None,
    input_type=gateway_dot_account__pb2._LISTACCOUNTSREQUEST,
    output_type=gateway_dot_account__pb2._LISTACCOUNTSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateAccount',
    full_name='gateway.Gateway.UpdateAccount',
    index=5,
    containing_service=None,
    input_type=gateway_dot_account__pb2._UPDATEACCOUNTREQUEST,
    output_type=gateway_dot_account__pb2._ACCOUNT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteAccount',
    full_name='gateway.Gateway.DeleteAccount',
    index=6,
    containing_service=None,
    input_type=gateway_dot_account__pb2._DELETEACCOUNTREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CreateBatchJob',
    full_name='gateway.Gateway.CreateBatchJob',
    index=7,
    containing_service=None,
    input_type=gateway_dot_batch__job__pb2._CREATEBATCHJOBREQUEST,
    output_type=gateway_dot_batch__job__pb2._BATCHJOB,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetBatchJob',
    full_name='gateway.Gateway.GetBatchJob',
    index=8,
    containing_service=None,
    input_type=gateway_dot_batch__job__pb2._GETBATCHJOBREQUEST,
    output_type=gateway_dot_batch__job__pb2._BATCHJOB,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListBatchJobs',
    full_name='gateway.Gateway.ListBatchJobs',
    index=9,
    containing_service=None,
    input_type=gateway_dot_batch__job__pb2._LISTBATCHJOBSREQUEST,
    output_type=gateway_dot_batch__job__pb2._LISTBATCHJOBSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteBatchJob',
    full_name='gateway.Gateway.DeleteBatchJob',
    index=10,
    containing_service=None,
    input_type=gateway_dot_batch__job__pb2._DELETEBATCHJOBREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='BatchDeleteBatchJobs',
    full_name='gateway.Gateway.BatchDeleteBatchJobs',
    index=11,
    containing_service=None,
    input_type=gateway_dot_batch__job__pb2._BATCHDELETEBATCHJOBSREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CancelBatchJob',
    full_name='gateway.Gateway.CancelBatchJob',
    index=12,
    containing_service=None,
    input_type=gateway_dot_batch__job__pb2._CANCELBATCHJOBREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CreateCluster',
    full_name='gateway.Gateway.CreateCluster',
    index=13,
    containing_service=None,
    input_type=gateway_dot_cluster__pb2._CREATECLUSTERREQUEST,
    output_type=gateway_dot_cluster__pb2._CLUSTER,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetCluster',
    full_name='gateway.Gateway.GetCluster',
    index=14,
    containing_service=None,
    input_type=gateway_dot_cluster__pb2._GETCLUSTERREQUEST,
    output_type=gateway_dot_cluster__pb2._CLUSTER,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListClusters',
    full_name='gateway.Gateway.ListClusters',
    index=15,
    containing_service=None,
    input_type=gateway_dot_cluster__pb2._LISTCLUSTERSREQUEST,
    output_type=gateway_dot_cluster__pb2._LISTCLUSTERSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateCluster',
    full_name='gateway.Gateway.UpdateCluster',
    index=16,
    containing_service=None,
    input_type=gateway_dot_cluster__pb2._UPDATECLUSTERREQUEST,
    output_type=gateway_dot_cluster__pb2._CLUSTER,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteCluster',
    full_name='gateway.Gateway.DeleteCluster',
    index=17,
    containing_service=None,
    input_type=gateway_dot_cluster__pb2._DELETECLUSTERREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetClusterConnectionInfo',
    full_name='gateway.Gateway.GetClusterConnectionInfo',
    index=18,
    containing_service=None,
    input_type=gateway_dot_cluster__pb2._GETCLUSTERCONNECTIONINFOREQUEST,
    output_type=gateway_dot_cluster__pb2._CLUSTERCONNECTIONINFO,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CreateEnvironment',
    full_name='gateway.Gateway.CreateEnvironment',
    index=19,
    containing_service=None,
    input_type=gateway_dot_environment__pb2._CREATEENVIRONMENTREQUEST,
    output_type=gateway_dot_environment__pb2._ENVIRONMENT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetEnvironment',
    full_name='gateway.Gateway.GetEnvironment',
    index=20,
    containing_service=None,
    input_type=gateway_dot_environment__pb2._GETENVIRONMENTREQUEST,
    output_type=gateway_dot_environment__pb2._ENVIRONMENT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListEnvironments',
    full_name='gateway.Gateway.ListEnvironments',
    index=21,
    containing_service=None,
    input_type=gateway_dot_environment__pb2._LISTENVIRONMENTSREQUEST,
    output_type=gateway_dot_environment__pb2._LISTENVIRONMENTSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateEnvironment',
    full_name='gateway.Gateway.UpdateEnvironment',
    index=22,
    containing_service=None,
    input_type=gateway_dot_environment__pb2._UPDATEENVIRONMENTREQUEST,
    output_type=gateway_dot_environment__pb2._ENVIRONMENT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteEnvironment',
    full_name='gateway.Gateway.DeleteEnvironment',
    index=23,
    containing_service=None,
    input_type=gateway_dot_environment__pb2._DELETEENVIRONMENTREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='BatchDeleteEnvironments',
    full_name='gateway.Gateway.BatchDeleteEnvironments',
    index=24,
    containing_service=None,
    input_type=gateway_dot_environment__pb2._BATCHDELETEENVIRONMENTSREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ConnectEnvironment',
    full_name='gateway.Gateway.ConnectEnvironment',
    index=25,
    containing_service=None,
    input_type=gateway_dot_environment__pb2._CONNECTENVIRONMENTREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DisconnectEnvironment',
    full_name='gateway.Gateway.DisconnectEnvironment',
    index=26,
    containing_service=None,
    input_type=gateway_dot_environment__pb2._DISCONNECTENVIRONMENTREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CreateSnapshot',
    full_name='gateway.Gateway.CreateSnapshot',
    index=27,
    containing_service=None,
    input_type=gateway_dot_snapshot__pb2._CREATESNAPSHOTREQUEST,
    output_type=gateway_dot_snapshot__pb2._SNAPSHOT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetSnapshot',
    full_name='gateway.Gateway.GetSnapshot',
    index=28,
    containing_service=None,
    input_type=gateway_dot_snapshot__pb2._GETSNAPSHOTREQUEST,
    output_type=gateway_dot_snapshot__pb2._SNAPSHOT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListSnapshots',
    full_name='gateway.Gateway.ListSnapshots',
    index=29,
    containing_service=None,
    input_type=gateway_dot_snapshot__pb2._LISTSNAPSHOTSREQUEST,
    output_type=gateway_dot_snapshot__pb2._LISTSNAPSHOTSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteSnapshot',
    full_name='gateway.Gateway.DeleteSnapshot',
    index=30,
    containing_service=None,
    input_type=gateway_dot_snapshot__pb2._DELETESNAPSHOTREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetBatchJobLogs',
    full_name='gateway.Gateway.GetBatchJobLogs',
    index=31,
    containing_service=None,
    input_type=gateway_dot_batch__job__pb2._GETBATCHJOBLOGSREQUEST,
    output_type=gateway_dot_batch__job__pb2._GETBATCHJOBLOGSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CreateNodePool',
    full_name='gateway.Gateway.CreateNodePool',
    index=32,
    containing_service=None,
    input_type=gateway_dot_node__pool__pb2._CREATENODEPOOLREQUEST,
    output_type=gateway_dot_node__pool__pb2._NODEPOOL,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetNodePool',
    full_name='gateway.Gateway.GetNodePool',
    index=33,
    containing_service=None,
    input_type=gateway_dot_node__pool__pb2._GETNODEPOOLREQUEST,
    output_type=gateway_dot_node__pool__pb2._NODEPOOL,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListNodePools',
    full_name='gateway.Gateway.ListNodePools',
    index=34,
    containing_service=None,
    input_type=gateway_dot_node__pool__pb2._LISTNODEPOOLSREQUEST,
    output_type=gateway_dot_node__pool__pb2._LISTNODEPOOLSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateNodePool',
    full_name='gateway.Gateway.UpdateNodePool',
    index=35,
    containing_service=None,
    input_type=gateway_dot_node__pool__pb2._UPDATENODEPOOLREQUEST,
    output_type=gateway_dot_node__pool__pb2._NODEPOOL,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteNodePool',
    full_name='gateway.Gateway.DeleteNodePool',
    index=36,
    containing_service=None,
    input_type=gateway_dot_node__pool__pb2._DELETENODEPOOLREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='BatchDeleteNodePools',
    full_name='gateway.Gateway.BatchDeleteNodePools',
    index=37,
    containing_service=None,
    input_type=gateway_dot_node__pool__pb2._BATCHDELETENODEPOOLSREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetNodePoolStats',
    full_name='gateway.Gateway.GetNodePoolStats',
    index=38,
    containing_service=None,
    input_type=gateway_dot_node__pool__pb2._GETNODEPOOLSTATSREQUEST,
    output_type=gateway_dot_node__pool__pb2._NODEPOOLSTATS,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CreateUser',
    full_name='gateway.Gateway.CreateUser',
    index=39,
    containing_service=None,
    input_type=gateway_dot_user__pb2._CREATEUSERREQUEST,
    output_type=gateway_dot_user__pb2._USER,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetUser',
    full_name='gateway.Gateway.GetUser',
    index=40,
    containing_service=None,
    input_type=gateway_dot_user__pb2._GETUSERREQUEST,
    output_type=gateway_dot_user__pb2._USER,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListUsers',
    full_name='gateway.Gateway.ListUsers',
    index=41,
    containing_service=None,
    input_type=gateway_dot_user__pb2._LISTUSERSREQUEST,
    output_type=gateway_dot_user__pb2._LISTUSERSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateUser',
    full_name='gateway.Gateway.UpdateUser',
    index=42,
    containing_service=None,
    input_type=gateway_dot_user__pb2._UPDATEUSERREQUEST,
    output_type=gateway_dot_user__pb2._USER,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteUser',
    full_name='gateway.Gateway.DeleteUser',
    index=43,
    containing_service=None,
    input_type=gateway_dot_user__pb2._DELETEUSERREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CreateModel',
    full_name='gateway.Gateway.CreateModel',
    index=44,
    containing_service=None,
    input_type=gateway_dot_model__pb2._CREATEMODELREQUEST,
    output_type=gateway_dot_model__pb2._MODEL,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetModel',
    full_name='gateway.Gateway.GetModel',
    index=45,
    containing_service=None,
    input_type=gateway_dot_model__pb2._GETMODELREQUEST,
    output_type=gateway_dot_model__pb2._MODEL,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListModels',
    full_name='gateway.Gateway.ListModels',
    index=46,
    containing_service=None,
    input_type=gateway_dot_model__pb2._LISTMODELSREQUEST,
    output_type=gateway_dot_model__pb2._LISTMODELSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateModel',
    full_name='gateway.Gateway.UpdateModel',
    index=47,
    containing_service=None,
    input_type=gateway_dot_model__pb2._UPDATEMODELREQUEST,
    output_type=gateway_dot_model__pb2._MODEL,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteModel',
    full_name='gateway.Gateway.DeleteModel',
    index=48,
    containing_service=None,
    input_type=gateway_dot_model__pb2._DELETEMODELREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetModelUploadEndpoint',
    full_name='gateway.Gateway.GetModelUploadEndpoint',
    index=49,
    containing_service=None,
    input_type=gateway_dot_model__pb2._GETMODELUPLOADENDPOINTREQUEST,
    output_type=gateway_dot_model__pb2._GETMODELUPLOADENDPOINTRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ValidateModelUpload',
    full_name='gateway.Gateway.ValidateModelUpload',
    index=50,
    containing_service=None,
    input_type=gateway_dot_model__pb2._VALIDATEMODELUPLOADREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeployModel',
    full_name='gateway.Gateway.DeployModel',
    index=51,
    containing_service=None,
    input_type=gateway_dot_model__pb2._DEPLOYMODELREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UndeployModel',
    full_name='gateway.Gateway.UndeployModel',
    index=52,
    containing_service=None,
    input_type=gateway_dot_model__pb2._UNDEPLOYMODELREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CreateDeployment',
    full_name='gateway.Gateway.CreateDeployment',
    index=53,
    containing_service=None,
    input_type=gateway_dot_deployment__pb2._CREATEDEPLOYMENTREQUEST,
    output_type=gateway_dot_deployment__pb2._DEPLOYMENT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetDeployment',
    full_name='gateway.Gateway.GetDeployment',
    index=54,
    containing_service=None,
    input_type=gateway_dot_deployment__pb2._GETDEPLOYMENTREQUEST,
    output_type=gateway_dot_deployment__pb2._DEPLOYMENT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StartDeployment',
    full_name='gateway.Gateway.StartDeployment',
    index=55,
    containing_service=None,
    input_type=gateway_dot_deployment__pb2._STARTDEPLOYMENTREQUEST,
    output_type=gateway_dot_deployment__pb2._DEPLOYMENT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListDeployments',
    full_name='gateway.Gateway.ListDeployments',
    index=56,
    containing_service=None,
    input_type=gateway_dot_deployment__pb2._LISTDEPLOYMENTSREQUEST,
    output_type=gateway_dot_deployment__pb2._LISTDEPLOYMENTSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateDeployment',
    full_name='gateway.Gateway.UpdateDeployment',
    index=57,
    containing_service=None,
    input_type=gateway_dot_deployment__pb2._UPDATEDEPLOYMENTREQUEST,
    output_type=gateway_dot_deployment__pb2._DEPLOYMENT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteDeployment',
    full_name='gateway.Gateway.DeleteDeployment',
    index=58,
    containing_service=None,
    input_type=gateway_dot_deployment__pb2._DELETEDEPLOYMENTREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CreateAwsIamRoleBinding',
    full_name='gateway.Gateway.CreateAwsIamRoleBinding',
    index=59,
    containing_service=None,
    input_type=gateway_dot_aws__iam__role__binding__pb2._CREATEAWSIAMROLEBINDINGREQUEST,
    output_type=gateway_dot_aws__iam__role__binding__pb2._AWSIAMROLEBINDING,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListAwsIamRoleBindings',
    full_name='gateway.Gateway.ListAwsIamRoleBindings',
    index=60,
    containing_service=None,
    input_type=gateway_dot_aws__iam__role__binding__pb2._LISTAWSIAMROLEBINDINGSREQUEST,
    output_type=gateway_dot_aws__iam__role__binding__pb2._LISTAWSIAMROLEBINDINGSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteAwsIamRoleBinding',
    full_name='gateway.Gateway.DeleteAwsIamRoleBinding',
    index=61,
    containing_service=None,
    input_type=gateway_dot_aws__iam__role__binding__pb2._DELETEAWSIAMROLEBINDINGREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ReconcileCluster',
    full_name='gateway.Gateway.ReconcileCluster',
    index=62,
    containing_service=None,
    input_type=gateway_dot_cluster__pb2._RECONCILECLUSTERREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ReconcileNodePool',
    full_name='gateway.Gateway.ReconcileNodePool',
    index=63,
    containing_service=None,
    input_type=gateway_dot_node__pool__pb2._RECONCILENODEPOOLREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_GATEWAY)

DESCRIPTOR.services_by_name['Gateway'] = _GATEWAY

# @@protoc_insertion_point(module_scope)



import httpx

RAW_RESPONSE_HEADER = "X-Stainless-Raw-Response"
STREAMED_RAW_RESPONSE_HEADER = "X-Stainless-Streamed-Raw-Response"

# default timeout is 10 minutes
DEFAULT_MAX_RETRIES = 2
DEFAULT_LIMITS = httpx.Limits(max_connections=100, max_keepalive_connections=20)



import json
import httpx
from typing import Dict, Any, Optional, Generator, Union, AsyncGenerator
import httpx_sse
import fireworks.client
from .error import (
    AuthenticationError,
    BadGatewayError,
    InternalServerError,
    InvalidRequestError,
    PermissionError,
    RateLimitError,
    ServiceUnavailableError,
)


# Helper functions for api key and base url prevent cyclic dependencies
def default_api_key() -> str:
    if fireworks.client.api_key is not None:
        return fireworks.client.api_key
    else:
        raise ValueError(
            "No API key provided. You can set your API key in code using 'fireworks.client.api_key = <API-KEY>', or you can set the environment variable FIREWORKS_API_KEY=<API-KEY>)."
        )


def default_base_url() -> str:
    return fireworks.client.base_url


class FireworksClient:
    """
    Fireworks client class to help with request handling for
    - get
    - post
      - with & without async
      - with & without streaming
    """

    def __init__(
        self,
        request_timeout=600,
        *,
        api_key: Union[str, None] = None,
        base_url: Union[str, httpx.URL, None] = None,
        extra_headers: Optional[Dict[str, str]] = None,
        **kwargs,
    ) -> None:
        """Initializes the Fireworks client.

        Args:
          request_timeout (int): A request timeout in seconds.
        """
        if "request_timeout" in kwargs:
            request_timeout = kwargs["request_timeout"]
        self.api_key = api_key or default_api_key()
        if not self.api_key:
            raise AuthenticationError(
                "No API key provided. You can set your API key in code using 'fireworks.client.api_key = <API-KEY>', or you can set the environment variable FIREWORKS_API_KEY=<API-KEY>)."
            )
        self.base_url = base_url or default_base_url()
        self.request_timeout = request_timeout
        self.client_kwargs = kwargs

        self._client: httpx.Client = self.create_client(extra_headers=extra_headers)
        self._async_client: httpx.AsyncClient = self.create_async_client(
            extra_headers=extra_headers
        )

    def _raise_for_status(self, resp):
        # Function to get error message or default to response code name
        def get_error_message(response, error_type: str = "invalid_request_error"):
            try:
                # Try to return the JSON body
                return response.json()
            except json.decoder.JSONDecodeError:
                # If JSON parsing fails, return the HTTP status code name
                return {
                    "error": {
                        "object": "error",
                        "type": error_type,
                        "message": response.reason_phrase,
                    }
                }

        if resp.status_code == 400:
            raise InvalidRequestError(get_error_message(resp))
        elif resp.status_code == 401:
            raise AuthenticationError(get_error_message(resp))
        elif resp.status_code == 403:
            raise PermissionError(get_error_message(resp))
        elif resp.status_code == 404:
            raise InvalidRequestError(get_error_message(resp))
        elif resp.status_code == 429:
            raise RateLimitError(get_error_message(resp))
        elif resp.status_code == 500:
            raise InternalServerError(get_error_message(resp, "internal_server_error"))
        elif resp.status_code == 502:
            raise BadGatewayError(get_error_message(resp, "internal_server_error"))
        elif resp.status_code == 503:
            raise ServiceUnavailableError(
                get_error_message(resp, "internal_server_error")
            )
        else:
            resp.raise_for_status()

    async def _async_error_handling(self, resp):
        if resp.is_error:
            await resp.aread()
        self._raise_for_status(resp)

    def _error_handling(self, resp):
        if resp.is_error:
            resp.read()
        self._raise_for_status(resp)

    def _get_request(self, url: str) -> Dict[str, Any]:
        resp = self._client.get(url)
        self._error_handling(resp)
        return resp.json()

    def _get_headers(self, extra_headers: Optional[Dict[str, str]]) -> Dict[str, str]:
        if extra_headers:
            return {**self._client.headers, **extra_headers}
        return self._client.headers

    def post_request_streaming(
        self,
        url: str,
        data: Optional[Dict[str, Any]] = None,
        extra_headers: Optional[Dict[str, str]] = None,
    ) -> Generator[str, None, None]:
        with httpx_sse.connect_sse(
            self._client,
            url=url,
            method="POST",
            json=data,
            headers=self._get_headers(extra_headers),
        ) as event_source:
            self._error_handling(event_source.response)
            for sse in event_source.iter_sse():
                yield sse.data

    def post_request_non_streaming(
        self,
        url: str,
        data: Optional[Dict[str, Any]] = None,
        extra_headers: Optional[Dict[str, str]] = None,
    ) -> Dict[str, Any]:
        response = self._client.post(
            url, json=data, headers=self._get_headers(extra_headers)
        )
        self._error_handling(response)
        return response.json()

    async def post_request_async_streaming(
        self,
        url: str,
        data: Optional[Dict[str, Any]] = None,
        extra_headers: Optional[Dict[str, str]] = None,
    ) -> AsyncGenerator[str, None]:
        async with httpx_sse.aconnect_sse(
            self._async_client,
            url=url,
            method="POST",
            json=data,
            headers=self._get_headers(extra_headers),
        ) as event_source:
            await self._async_error_handling(event_source.response)
            async for sse in event_source.aiter_sse():
                yield sse.data

    async def post_request_async_non_streaming(
        self,
        url: str,
        data: Optional[Dict[str, Any]] = None,
        extra_headers: Optional[Dict[str, str]] = None,
    ) -> Dict[str, Any]:
        response = await self._async_client.post(
            url, json=data, headers=self._get_headers(extra_headers)
        )
        await self._async_error_handling(response)
        return response.json()

    def __enter__(self):
        return self

    def __exit__(self, *exc):
        self.close()

    def close(self):
        self._client.close()

    async def __aenter__(self):
        return self

    async def __aexit__(self, *exc):
        await self.aclose()

    async def aclose(self):
        await self._async_client.aclose()

    def create_client(
        self, extra_headers: Optional[Dict[str, str]] = None
    ) -> httpx.Client:
        headers = {"Authorization": f"Bearer {self.api_key}"}
        if extra_headers:
            headers = {**headers, **extra_headers}
        return httpx.Client(
            headers=headers,
            timeout=self.request_timeout,
            **self.client_kwargs,
        )

    def create_async_client(
        self, extra_headers: Optional[Dict[str, str]] = None
    ) -> httpx.AsyncClient:
        headers = {"Authorization": f"Bearer {self.api_key}"}
        if extra_headers:
            headers = {**headers, **extra_headers}
        return httpx.AsyncClient(
            headers=headers,
            timeout=self.request_timeout,
            **self.client_kwargs,
        )



import logging

logger = logging.getLogger("fireworks")

_console_log_level: int = logging.INFO

_NAME_TO_LEVEL = {
    "CRITICAL": logging.CRITICAL,
    "FATAL": logging.FATAL,
    "ERROR": logging.ERROR,
    "WARN": logging.WARNING,
    "WARNING": logging.WARNING,
    "INFO": logging.INFO,
    "DEBUG": logging.DEBUG,
}


def set_console_log_level(level: str) -> None:
    """
    Controls console logging.

    Args:
        level: the minimum level that prints out to console.
            Supported values: [CRITICAL, FATAL, ERROR, WARN,
            WARNING, INFO, DEBUG]
    """
    _console_log_level = _NAME_TO_LEVEL.get(level)
    if _console_log_level is None:
        raise ValueError(
            f"unrecognized log level {level}. Supported "
            f"values are {list(_NAME_TO_LEVEL.keys())}"
        )


def _log(message: str, level: int) -> None:
    """
    Logs message with a specific severity level.

    Args:
        message: the message to log,
        level: the severity level.
    """
    if _console_log_level >= level:
        print(message)
    logger.log(level, message)


def log_debug(message: str) -> None:
    _log(message, logging.DEBUG)


def log_info(message: str) -> None:
    _log(message, logging.INFO)


def log_warning(message: str) -> None:
    _log(message, logging.WARNING)


def log_error(message: str) -> None:
    _log(message, logging.ERROR)


def log_fatal(message: str) -> None:
    _log(message, logging.FATAL)


def log_critical(message: str) -> None:
    _log(message, logging.CRITICAL)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gateway/environment.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import status_pb2 as gateway_dot_status__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='gateway/environment.proto',
  package='gateway',
  syntax='proto3',
  serialized_options=b'Z\n./;gateway',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x19gateway/environment.proto\x12\x07gateway\x1a\x14gateway/status.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xa2\x05\n\x0b\x45nvironment\x12\x14\n\x04name\x18\x01 \x01(\tB\x06\xe0\x41\x03\xe0\x41\x05\x12\x19\n\x0c\x64isplay_name\x18\x03 \x01(\tB\x03\xe0\x41\x01\x12\x37\n\x0b\x63reate_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x06\xe0\x41\x03\xe0\x41\x05\x12\x1a\n\ncreated_by\x18\x06 \x01(\tB\x06\xe0\x41\x03\xe0\x41\x05\x12\x31\n\x05state\x18\n \x01(\x0e\x32\x1a.gateway.Environment.StateB\x06\xe0\x41\x03\xe0\x41\x05\x12\'\n\x06status\x18\x0b \x01(\x0b\x32\x0f.gateway.StatusB\x06\xe0\x41\x03\xe0\x41\x05\x12:\n\nconnection\x18\x02 \x01(\x0b\x32\x1e.gateway.EnvironmentConnectionB\x06\xe0\x41\x03\xe0\x41\x05\x12\x1e\n\x0e\x62\x61se_image_ref\x18\x07 \x01(\tB\x06\xe0\x41\x01\xe0\x41\x05\x12\x19\n\timage_ref\x18\x04 \x01(\tB\x06\xe0\x41\x03\xe0\x41\x05\x12\"\n\x12snapshot_image_ref\x18\x08 \x01(\tB\x06\xe0\x41\x03\xe0\x41\x05\x12\x0e\n\x06shared\x18\x0c \x01(\x08\x12?\n\x0b\x61nnotations\x18\t \x03(\x0b\x32%.gateway.Environment.AnnotationsEntryB\x03\xe0\x41\x01\x1a\x32\n\x10\x41nnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x90\x01\n\x05State\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\x0c\n\x08\x43REATING\x10\x01\x12\x10\n\x0c\x44ISCONNECTED\x10\x02\x12\x0e\n\nCONNECTING\x10\x03\x12\r\n\tCONNECTED\x10\x04\x12\x11\n\rDISCONNECTING\x10\x05\x12\x10\n\x0cRECONNECTING\x10\x07\x12\x0c\n\x08\x44\x45LETING\x10\x06\"\x93\x01\n\x15\x45nvironmentConnection\x12\x19\n\x0cnode_pool_id\x18\x01 \x01(\tB\x03\xe0\x41\x02\x12\x16\n\tnum_ranks\x18\x04 \x01(\x05\x42\x03\xe0\x41\x01\x12\x11\n\x04role\x18\x06 \x01(\tB\x03\xe0\x41\x01\x12\x14\n\x04zone\x18\x05 \x01(\tB\x06\xe0\x41\x03\xe0\x41\x05\x12\x1e\n\x11use_local_storage\x18\x07 \x01(\x08\x42\x03\xe0\x41\x01\"|\n\x18\x43reateEnvironmentRequest\x12\x13\n\x06parent\x18\x01 \x01(\tB\x03\xe0\x41\x02\x12.\n\x0b\x65nvironment\x18\x02 \x01(\x0b\x32\x14.gateway.EnvironmentB\x03\xe0\x41\x02\x12\x1b\n\x0e\x65nvironment_id\x18\x03 \x01(\tB\x03\xe0\x41\x02\"*\n\x15GetEnvironmentRequest\x12\x11\n\x04name\x18\x01 \x01(\tB\x03\xe0\x41\x02\"\x8b\x01\n\x17ListEnvironmentsRequest\x12\x13\n\x06parent\x18\x01 \x01(\tB\x03\xe0\x41\x02\x12\x16\n\tpage_size\x18\x02 \x01(\x05\x42\x03\xe0\x41\x01\x12\x17\n\npage_token\x18\x03 \x01(\tB\x03\xe0\x41\x01\x12\x13\n\x06\x66ilter\x18\x04 \x01(\tB\x03\xe0\x41\x01\x12\x15\n\x08order_by\x18\x05 \x01(\tB\x03\xe0\x41\x01\"s\n\x18ListEnvironmentsResponse\x12*\n\x0c\x65nvironments\x18\x01 \x03(\x0b\x32\x14.gateway.Environment\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\x12\x12\n\ntotal_size\x18\x03 \x01(\x05\"J\n\x18UpdateEnvironmentRequest\x12.\n\x0b\x65nvironment\x18\x01 \x01(\x0b\x32\x14.gateway.EnvironmentB\x03\xe0\x41\x02\"-\n\x18\x44\x65leteEnvironmentRequest\x12\x11\n\x04name\x18\x01 \x01(\tB\x03\xe0\x41\x02\"I\n\x1e\x42\x61tchDeleteEnvironmentsRequest\x12\x13\n\x06parent\x18\x01 \x01(\tB\x03\xe0\x41\x02\x12\x12\n\x05names\x18\x02 \x03(\tB\x03\xe0\x41\x02\"\x84\x01\n\x19\x43onnectEnvironmentRequest\x12\x11\n\x04name\x18\x01 \x01(\tB\x03\xe0\x41\x02\x12\x37\n\nconnection\x18\x02 \x01(\x0b\x32\x1e.gateway.EnvironmentConnectionB\x03\xe0\x41\x02\x12\x1b\n\x0evscode_version\x18\x03 \x01(\tB\x03\xe0\x41\x01\"c\n\x1c\x44isconnectEnvironmentRequest\x12\x11\n\x04name\x18\x01 \x01(\tB\x03\xe0\x41\x02\x12\x12\n\x05\x66orce\x18\x02 \x01(\x08\x42\x03\xe0\x41\x01\x12\x1c\n\x0freset_snapshots\x18\x03 \x01(\x08\x42\x03\xe0\x41\x01\x42\x0cZ\n./;gatewayb\x06proto3'
  ,
  dependencies=[gateway_dot_status__pb2.DESCRIPTOR,google_dot_api_dot_field__behavior__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])



_ENVIRONMENT_STATE = _descriptor.EnumDescriptor(
  name='State',
  full_name='gateway.Environment.State',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='STATE_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CREATING', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DISCONNECTED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CONNECTING', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CONNECTED', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DISCONNECTING', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RECONNECTING', index=6, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DELETING', index=7, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=657,
  serialized_end=801,
)
_sym_db.RegisterEnumDescriptor(_ENVIRONMENT_STATE)


_ENVIRONMENT_ANNOTATIONSENTRY = _descriptor.Descriptor(
  name='AnnotationsEntry',
  full_name='gateway.Environment.AnnotationsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='gateway.Environment.AnnotationsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='gateway.Environment.AnnotationsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=604,
  serialized_end=654,
)

_ENVIRONMENT = _descriptor.Descriptor(
  name='Environment',
  full_name='gateway.Environment',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='gateway.Environment.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\003\340A\005', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='display_name', full_name='gateway.Environment.display_name', index=1,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='create_time', full_name='gateway.Environment.create_time', index=2,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\003\340A\005', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='created_by', full_name='gateway.Environment.created_by', index=3,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\003\340A\005', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='state', full_name='gateway.Environment.state', index=4,
      number=10, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\003\340A\005', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status', full_name='gateway.Environment.status', index=5,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\003\340A\005', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='connection', full_name='gateway.Environment.connection', index=6,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\003\340A\005', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='base_image_ref', full_name='gateway.Environment.base_image_ref', index=7,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\001\340A\005', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='image_ref', full_name='gateway.Environment.image_ref', index=8,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\003\340A\005', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='snapshot_image_ref', full_name='gateway.Environment.snapshot_image_ref', index=9,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\003\340A\005', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='shared', full_name='gateway.Environment.shared', index=10,
      number=12, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='annotations', full_name='gateway.Environment.annotations', index=11,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_ENVIRONMENT_ANNOTATIONSENTRY, ],
  enum_types=[
    _ENVIRONMENT_STATE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=127,
  serialized_end=801,
)


_ENVIRONMENTCONNECTION = _descriptor.Descriptor(
  name='EnvironmentConnection',
  full_name='gateway.EnvironmentConnection',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='node_pool_id', full_name='gateway.EnvironmentConnection.node_pool_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\002', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_ranks', full_name='gateway.EnvironmentConnection.num_ranks', index=1,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='role', full_name='gateway.EnvironmentConnection.role', index=2,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='zone', full_name='gateway.EnvironmentConnection.zone', index=3,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\003\340A\005', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='use_local_storage', full_name='gateway.EnvironmentConnection.use_local_storage', index=4,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=804,
  serialized_end=951,
)


_CREATEENVIRONMENTREQUEST = _descriptor.Descriptor(
  name='CreateEnvironmentRequest',
  full_name='gateway.CreateEnvironmentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='parent', full_name='gateway.CreateEnvironmentRequest.parent', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\002', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='environment', full_name='gateway.CreateEnvironmentRequest.environment', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\002', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='environment_id', full_name='gateway.CreateEnvironmentRequest.environment_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\002', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=953,
  serialized_end=1077,
)


_GETENVIRONMENTREQUEST = _descriptor.Descriptor(
  name='GetEnvironmentRequest',
  full_name='gateway.GetEnvironmentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='gateway.GetEnvironmentRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\002', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1079,
  serialized_end=1121,
)


_LISTENVIRONMENTSREQUEST = _descriptor.Descriptor(
  name='ListEnvironmentsRequest',
  full_name='gateway.ListEnvironmentsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='parent', full_name='gateway.ListEnvironmentsRequest.parent', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\002', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='page_size', full_name='gateway.ListEnvironmentsRequest.page_size', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='page_token', full_name='gateway.ListEnvironmentsRequest.page_token', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='filter', full_name='gateway.ListEnvironmentsRequest.filter', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='order_by', full_name='gateway.ListEnvironmentsRequest.order_by', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1124,
  serialized_end=1263,
)


_LISTENVIRONMENTSRESPONSE = _descriptor.Descriptor(
  name='ListEnvironmentsResponse',
  full_name='gateway.ListEnvironmentsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='environments', full_name='gateway.ListEnvironmentsResponse.environments', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='next_page_token', full_name='gateway.ListEnvironmentsResponse.next_page_token', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='total_size', full_name='gateway.ListEnvironmentsResponse.total_size', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1265,
  serialized_end=1380,
)


_UPDATEENVIRONMENTREQUEST = _descriptor.Descriptor(
  name='UpdateEnvironmentRequest',
  full_name='gateway.UpdateEnvironmentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='environment', full_name='gateway.UpdateEnvironmentRequest.environment', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\002', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1382,
  serialized_end=1456,
)


_DELETEENVIRONMENTREQUEST = _descriptor.Descriptor(
  name='DeleteEnvironmentRequest',
  full_name='gateway.DeleteEnvironmentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='gateway.DeleteEnvironmentRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\002', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1458,
  serialized_end=1503,
)


_BATCHDELETEENVIRONMENTSREQUEST = _descriptor.Descriptor(
  name='BatchDeleteEnvironmentsRequest',
  full_name='gateway.BatchDeleteEnvironmentsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='parent', full_name='gateway.BatchDeleteEnvironmentsRequest.parent', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\002', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='names', full_name='gateway.BatchDeleteEnvironmentsRequest.names', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\002', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1505,
  serialized_end=1578,
)


_CONNECTENVIRONMENTREQUEST = _descriptor.Descriptor(
  name='ConnectEnvironmentRequest',
  full_name='gateway.ConnectEnvironmentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='gateway.ConnectEnvironmentRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\002', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='connection', full_name='gateway.ConnectEnvironmentRequest.connection', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\002', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='vscode_version', full_name='gateway.ConnectEnvironmentRequest.vscode_version', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1581,
  serialized_end=1713,
)


_DISCONNECTENVIRONMENTREQUEST = _descriptor.Descriptor(
  name='DisconnectEnvironmentRequest',
  full_name='gateway.DisconnectEnvironmentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='gateway.DisconnectEnvironmentRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\002', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='force', full_name='gateway.DisconnectEnvironmentRequest.force', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='reset_snapshots', full_name='gateway.DisconnectEnvironmentRequest.reset_snapshots', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\340A\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1715,
  serialized_end=1814,
)

_ENVIRONMENT_ANNOTATIONSENTRY.containing_type = _ENVIRONMENT
_ENVIRONMENT.fields_by_name['create_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ENVIRONMENT.fields_by_name['state'].enum_type = _ENVIRONMENT_STATE
_ENVIRONMENT.fields_by_name['status'].message_type = gateway_dot_status__pb2._STATUS
_ENVIRONMENT.fields_by_name['connection'].message_type = _ENVIRONMENTCONNECTION
_ENVIRONMENT.fields_by_name['annotations'].message_type = _ENVIRONMENT_ANNOTATIONSENTRY
_ENVIRONMENT_STATE.containing_type = _ENVIRONMENT
_CREATEENVIRONMENTREQUEST.fields_by_name['environment'].message_type = _ENVIRONMENT
_LISTENVIRONMENTSRESPONSE.fields_by_name['environments'].message_type = _ENVIRONMENT
_UPDATEENVIRONMENTREQUEST.fields_by_name['environment'].message_type = _ENVIRONMENT
_CONNECTENVIRONMENTREQUEST.fields_by_name['connection'].message_type = _ENVIRONMENTCONNECTION
DESCRIPTOR.message_types_by_name['Environment'] = _ENVIRONMENT
DESCRIPTOR.message_types_by_name['EnvironmentConnection'] = _ENVIRONMENTCONNECTION
DESCRIPTOR.message_types_by_name['CreateEnvironmentRequest'] = _CREATEENVIRONMENTREQUEST
DESCRIPTOR.message_types_by_name['GetEnvironmentRequest'] = _GETENVIRONMENTREQUEST
DESCRIPTOR.message_types_by_name['ListEnvironmentsRequest'] = _LISTENVIRONMENTSREQUEST
DESCRIPTOR.message_types_by_name['ListEnvironmentsResponse'] = _LISTENVIRONMENTSRESPONSE
DESCRIPTOR.message_types_by_name['UpdateEnvironmentRequest'] = _UPDATEENVIRONMENTREQUEST
DESCRIPTOR.message_types_by_name['DeleteEnvironmentRequest'] = _DELETEENVIRONMENTREQUEST
DESCRIPTOR.message_types_by_name['BatchDeleteEnvironmentsRequest'] = _BATCHDELETEENVIRONMENTSREQUEST
DESCRIPTOR.message_types_by_name['ConnectEnvironmentRequest'] = _CONNECTENVIRONMENTREQUEST
DESCRIPTOR.message_types_by_name['DisconnectEnvironmentRequest'] = _DISCONNECTENVIRONMENTREQUEST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Environment = _reflection.GeneratedProtocolMessageType('Environment', (_message.Message,), {

  'AnnotationsEntry' : _reflection.GeneratedProtocolMessageType('AnnotationsEntry', (_message.Message,), {
    'DESCRIPTOR' : _ENVIRONMENT_ANNOTATIONSENTRY,
    '__module__' : 'gateway.environment_pb2'
    # @@protoc_insertion_point(class_scope:gateway.Environment.AnnotationsEntry)
    })
  ,
  'DESCRIPTOR' : _ENVIRONMENT,
  '__module__' : 'gateway.environment_pb2'
  # @@protoc_insertion_point(class_scope:gateway.Environment)
  })
_sym_db.RegisterMessage(Environment)
_sym_db.RegisterMessage(Environment.AnnotationsEntry)

EnvironmentConnection = _reflection.GeneratedProtocolMessageType('EnvironmentConnection', (_message.Message,), {
  'DESCRIPTOR' : _ENVIRONMENTCONNECTION,
  '__module__' : 'gateway.environment_pb2'
  # @@protoc_insertion_point(class_scope:gateway.EnvironmentConnection)
  })
_sym_db.RegisterMessage(EnvironmentConnection)

CreateEnvironmentRequest = _reflection.GeneratedProtocolMessageType('CreateEnvironmentRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEENVIRONMENTREQUEST,
  '__module__' : 'gateway.environment_pb2'
  # @@protoc_insertion_point(class_scope:gateway.CreateEnvironmentRequest)
  })
_sym_db.RegisterMessage(CreateEnvironmentRequest)

GetEnvironmentRequest = _reflection.GeneratedProtocolMessageType('GetEnvironmentRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETENVIRONMENTREQUEST,
  '__module__' : 'gateway.environment_pb2'
  # @@protoc_insertion_point(class_scope:gateway.GetEnvironmentRequest)
  })
_sym_db.RegisterMessage(GetEnvironmentRequest)

ListEnvironmentsRequest = _reflection.GeneratedProtocolMessageType('ListEnvironmentsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTENVIRONMENTSREQUEST,
  '__module__' : 'gateway.environment_pb2'
  # @@protoc_insertion_point(class_scope:gateway.ListEnvironmentsRequest)
  })
_sym_db.RegisterMessage(ListEnvironmentsRequest)

ListEnvironmentsResponse = _reflection.GeneratedProtocolMessageType('ListEnvironmentsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTENVIRONMENTSRESPONSE,
  '__module__' : 'gateway.environment_pb2'
  # @@protoc_insertion_point(class_scope:gateway.ListEnvironmentsResponse)
  })
_sym_db.RegisterMessage(ListEnvironmentsResponse)

UpdateEnvironmentRequest = _reflection.GeneratedProtocolMessageType('UpdateEnvironmentRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEENVIRONMENTREQUEST,
  '__module__' : 'gateway.environment_pb2'
  # @@protoc_insertion_point(class_scope:gateway.UpdateEnvironmentRequest)
  })
_sym_db.RegisterMessage(UpdateEnvironmentRequest)

DeleteEnvironmentRequest = _reflection.GeneratedProtocolMessageType('DeleteEnvironmentRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEENVIRONMENTREQUEST,
  '__module__' : 'gateway.environment_pb2'
  # @@protoc_insertion_point(class_scope:gateway.DeleteEnvironmentRequest)
  })
_sym_db.RegisterMessage(DeleteEnvironmentRequest)

BatchDeleteEnvironmentsRequest = _reflection.GeneratedProtocolMessageType('BatchDeleteEnvironmentsRequest', (_message.Message,), {
  'DESCRIPTOR' : _BATCHDELETEENVIRONMENTSREQUEST,
  '__module__' : 'gateway.environment_pb2'
  # @@protoc_insertion_point(class_scope:gateway.BatchDeleteEnvironmentsRequest)
  })
_sym_db.RegisterMessage(BatchDeleteEnvironmentsRequest)

ConnectEnvironmentRequest = _reflection.GeneratedProtocolMessageType('ConnectEnvironmentRequest', (_message.Message,), {
  'DESCRIPTOR' : _CONNECTENVIRONMENTREQUEST,
  '__module__' : 'gateway.environment_pb2'
  # @@protoc_insertion_point(class_scope:gateway.ConnectEnvironmentRequest)
  })
_sym_db.RegisterMessage(ConnectEnvironmentRequest)

DisconnectEnvironmentRequest = _reflection.GeneratedProtocolMessageType('DisconnectEnvironmentRequest', (_message.Message,), {
  'DESCRIPTOR' : _DISCONNECTENVIRONMENTREQUEST,
  '__module__' : 'gateway.environment_pb2'
  # @@protoc_insertion_point(class_scope:gateway.DisconnectEnvironmentRequest)
  })
_sym_db.RegisterMessage(DisconnectEnvironmentRequest)


DESCRIPTOR._options = None
_ENVIRONMENT_ANNOTATIONSENTRY._options = None
_ENVIRONMENT.fields_by_name['name']._options = None
_ENVIRONMENT.fields_by_name['display_name']._options = None
_ENVIRONMENT.fields_by_name['create_time']._options = None
_ENVIRONMENT.fields_by_name['created_by']._options = None
_ENVIRONMENT.fields_by_name['state']._options = None
_ENVIRONMENT.fields_by_name['status']._options = None
_ENVIRONMENT.fields_by_name['connection']._options = None
_ENVIRONMENT.fields_by_name['base_image_ref']._options = None
_ENVIRONMENT.fields_by_name['image_ref']._options = None
_ENVIRONMENT.fields_by_name['snapshot_image_ref']._options = None
_ENVIRONMENT.fields_by_name['annotations']._options = None
_ENVIRONMENTCONNECTION.fields_by_name['node_pool_id']._options = None
_ENVIRONMENTCONNECTION.fields_by_name['num_ranks']._options = None
_ENVIRONMENTCONNECTION.fields_by_name['role']._options = None
_ENVIRONMENTCONNECTION.fields_by_name['zone']._options = None
_ENVIRONMENTCONNECTION.fields_by_name['use_local_storage']._options = None
_CREATEENVIRONMENTREQUEST.fields_by_name['parent']._options = None
_CREATEENVIRONMENTREQUEST.fields_by_name['environment']._options = None
_CREATEENVIRONMENTREQUEST.fields_by_name['environment_id']._options = None
_GETENVIRONMENTREQUEST.fields_by_name['name']._options = None
_LISTENVIRONMENTSREQUEST.fields_by_name['parent']._options = None
_LISTENVIRONMENTSREQUEST.fields_by_name['page_size']._options = None
_LISTENVIRONMENTSREQUEST.fields_by_name['page_token']._options = None
_LISTENVIRONMENTSREQUEST.fields_by_name['filter']._options = None
_LISTENVIRONMENTSREQUEST.fields_by_name['order_by']._options = None
_UPDATEENVIRONMENTREQUEST.fields_by_name['environment']._options = None
_DELETEENVIRONMENTREQUEST.fields_by_name['name']._options = None
_BATCHDELETEENVIRONMENTSREQUEST.fields_by_name['parent']._options = None
_BATCHDELETEENVIRONMENTSREQUEST.fields_by_name['names']._options = None
_CONNECTENVIRONMENTREQUEST.fields_by_name['name']._options = None
_CONNECTENVIRONMENTREQUEST.fields_by_name['connection']._options = None
_CONNECTENVIRONMENTREQUEST.fields_by_name['vscode_version']._options = None
_DISCONNECTENVIRONMENTREQUEST.fields_by_name['name']._options = None
_DISCONNECTENVIRONMENTREQUEST.fields_by_name['force']._options = None
_DISCONNECTENVIRONMENTREQUEST.fields_by_name['reset_snapshots']._options = None
# @@protoc_insertion_point(module_scope)



import os
from .completion import Completion
from .chat_completion import ChatCompletion
from .embedding import Embedding
from .rerank import Rerank
from .chat import Chat
from .model import Model
from .api_client_v2 import (
    Fireworks,
    AsyncFireworks,
)
from . import _version

__version__ = _version.get_versions()["version"]

api_key = os.environ.get("FIREWORKS_API_KEY")
base_url = os.environ.get("FIREWORKS_API_BASE", "https://api.fireworks.ai/inference/v1")

__all__ = [
    "__version__",
    "AsyncFireworks",
    "Chat",
    "ChatCompletion",
    "Completion",
    "Fireworks",
    "Model",
    "Embedding",
    "Rerank",
]



import torch
import enum
from contextlib import contextmanager
from dataclasses import dataclass
from typing import Dict, Optional
from safetensors.torch import save_model
import tempfile


class FluminaModule(torch.nn.Module):
    pass


def path(path: str):
    def wrapper(fn):
        fn._flumina_path = path
        return fn

    return wrapper


@dataclass
class _ExportedModule:
    serialized_weights: bytes
    path_to_method_name: Dict[str, str]


class _ExecMode(enum.Enum):
    NONE = 0
    EXPORT = 1
    SERVE = 2


_exec_mode = _ExecMode.NONE
_exported_module: Optional[_ExportedModule] = None


def _get_exported_module():
    global _exported_module
    em = _exported_module
    _exported_module = None
    return em


@contextmanager
def _with_exec_mode(mode):
    global _exec_mode
    old_mode = _exec_mode
    _exec_mode = mode
    try:
        yield
    finally:
        _exec_mode = old_mode


_initialized_module: Optional[FluminaModule] = None


def _get_initialized_module():
    global _initialized_module
    mod = _initialized_module
    _initialized_module = None
    return mod


def main(m: FluminaModule):
    if _exec_mode == _ExecMode.NONE:
        pass
    elif _exec_mode == _ExecMode.EXPORT:
        assert isinstance(
            m, FluminaModule
        ), "Argument to flumina.main() must be an instance of FluminaModule"
        path_to_method_name: Dict[str, str] = {}
        for k in dir(m):
            v = getattr(m, k)
            if hasattr(v, "_flumina_path"):
                path_to_method_name[v._flumina_path] = k

        with tempfile.NamedTemporaryFile() as ntf:
            save_model(m, ntf.name)
            with open(ntf.name, "rb") as f:
                serialized_weights = f.read()

        global _exported_module
        _exported_module = _ExportedModule(
            serialized_weights=serialized_weights,
            path_to_method_name=path_to_method_name,
        )
    elif _exec_mode == _ExecMode.SERVE:
        global _initialized_module
        _initialized_module = m
    else:
        raise NotImplementedError("Unsupported ExecMode", _exec_mode)



from typing import Dict, Optional

from .api import CompletionResponse, CompletionStreamResponse
from .base_completion import BaseCompletion
from .api_client import FireworksClient


class Completion(BaseCompletion):
    """
    Class for handling text completions.
    """

    endpoint = "completions"
    response_class = CompletionResponse
    stream_response_class = CompletionStreamResponse

    @classmethod
    def _get_data_key(cls) -> str:
        return "prompt"


class CompletionV2(Completion):
    """
    Class for handling text completions.
    """

    def __init__(self, client: FireworksClient):
        self._client = client

    def create(
        self,
        model,
        prompt=None,
        request_timeout=600,
        stream=False,
        extra_headers: Optional[Dict[str, str]] = None,
        **kwargs,
    ):
        return super().create(
            model,
            prompt,
            request_timeout,
            stream=stream,
            client=self._client,
            extra_headers=extra_headers,
            **kwargs,
        )

    def acreate(
        self,
        model,
        prompt=None,
        request_timeout=600,
        stream=True,
        extra_headers: Optional[Dict[str, str]] = None,
        **kwargs,
    ):
        return super().acreate(
            model,
            prompt,
            request_timeout,
            stream=stream,
            client=self._client,
            extra_headers=extra_headers,
            **kwargs,
        )



from typing import Optional, Type, List
import json
from pydantic import BaseModel
from .api_client import FireworksClient
from .error import InvalidRequestError
from .api import RerankResponse


class Rerank:
    """
    Base class for handling reranking, both synchronously and asynchronously

    Attributes:
      endpoint (str): API endpoint for the completion request.
      response_class (Type): Class used for parsing the non-streaming response.
    """

    endpoint = "rerank"  # Matching the API reference
    response_class = RerankResponse

    @classmethod
    def create(
        cls,
        model,
        query,
        documents: List[str],
        request_timeout=600,
        client: Optional[FireworksClient] = None,
        **kwargs,
    ):
        """
        Create a completion or chat completion.

        Args:
          model (str): Model name to use for the completion.
          input (str): The input string for embedding.
          request_timeout (int, optional): Request timeout in seconds. Defaults to 600.
          **kwargs: Additional keyword arguments.

        Returns:
          RerankResponse: results of reranking, with a relevant score for each document
        """
        kwargs["query"] = query
        kwargs["documents"] = documents
        return cls._create_non_streaming(
            model, request_timeout, client=client, **kwargs
        )

    @classmethod
    def acreate(
        cls,
        model,
        query,
        documents: List[str],
        request_timeout=600,
        client: Optional[FireworksClient] = None,
        **kwargs,
    ):
        """
        Asynchronously create a completion.

        Args:
          model (str): Model name to use for the completion.
          query (str): The input query string.
          documents (List(str)): The input string for documents.
          request_timeout (int, optional): Request timeout in seconds. Defaults to 600.
          **kwargs: Additional keyword arguments.

        Returns:
          AsyncGenerator[RerankResponse]: results of reranking, with a relevant score for each document
        """
        kwargs["query"] = query
        kwargs["documents"] = documents
        return cls._acreate_non_streaming(
            model, request_timeout, client=client, **kwargs
        )

    @classmethod
    def _create_non_streaming(
        cls,
        model: str,
        request_timeout=600,
        client: Optional[FireworksClient] = None,
        **kwargs,
    ):
        client = client or FireworksClient(request_timeout=request_timeout)
        data = {"model": model, **kwargs}

        print(f"{client.base_url}/{cls.endpoint}", data)
        response = client.post_request_non_streaming(
            f"{client.base_url}/{cls.endpoint}", data=data
        )
        return cls.response_class(**response)

    @classmethod
    async def _acreate_non_streaming(
        cls,
        model: str,
        request_timeout=600,
        client: Optional[FireworksClient] = None,
        **kwargs,
    ):
        client = client or FireworksClient(request_timeout=request_timeout)
        data = {"model": model, **kwargs}
        response = await client.post_request_async_non_streaming(
            f"{client.base_url}/{cls.endpoint}", data=data
        )
        return cls.response_class(**response)


class RerankV1(Rerank):
    def __init__(self, client: FireworksClient):
        self._client = client

    def create(
        self,
        model,
        query: str,
        documents: List[str],
        request_timeout=600,
        **kwargs,
    ):
        return super().create(
            model,
            query,
            documents,
            request_timeout,
            client=self._client,
            **kwargs,
        )

    def acreate(
        self,
        model,
        query: str,
        documents: List[str],
        request_timeout=600,
        **kwargs,
    ):
        return super().acreate(
            model,
            query,
            documents,
            request_timeout,
            client=self._client,
            **kwargs,
        )



import logging
import os

# ANSI escape sequences for different log levels
COLORS = {
    "WARNING": "\033[93m",  # Yellow
    "INFO": "\033[94m",  # Blue
    "DEBUG": "\033[92m",  # Green
    "CRITICAL": "\033[91m",  # Red
    "ERROR": "\033[91m",  # Red
    "RESET": "\033[0m",  # Reset to default
}


class ColoredFormatter(logging.Formatter):
    def format(self, record):
        # Format string with levelname and message
        log_fmt = f"%(asctime)s {COLORS['DEBUG']}Flumina Runtime: {COLORS[record.levelname]}%(message)s{COLORS['RESET']}"
        # ISO8601 format
        formatter = logging.Formatter(log_fmt, datefmt="%Y-%m-%dT%H:%M:%S")
        return formatter.format(record)


logger = None


def get_logger():
    global logger
    if logger is None:
        # Retrieve LOGLEVEL from environment or set default to 'INFO'
        log_level = os.getenv("LOGLEVEL", "INFO").upper()

        # Create a custom logger
        logger = logging.getLogger("flumina_logger")
        logger.setLevel(log_level)  # Set the log level
        logger.propagate = False

        # Check if handlers are already configured
        if not logger.handlers:
            # Create a stream handler with a custom formatter
            stream_handler = logging.StreamHandler()
            stream_handler.setFormatter(ColoredFormatter())

            # Add the stream handler to the custom logger
            logger.addHandler(stream_handler)

    return logger



from .module import (
    _with_exec_mode,
    _ExecMode,
    _get_exported_module,
    _get_initialized_module,
)

import builtins
from functools import partial

import torch
import os
import sys


ALLOWED_EXTERNAL_DEPS = set(sys.builtin_module_names).union(
    {
        "accelerate",
        "contextlib",
        "fireworks",
        "flux",
        "transformers",
        "diffusers",
        "torch",
        "torchaudio",
        "torchvision",
        "PIL",
        "pydantic",
        "io",
        "fastapi",
        "logging",
        "numpy",
        "pandas",
    }
)


def restricted_import(
    name,
    globals=None,
    locals=None,
    fromlist=(),
    level=0,
    orig_import=None,
):
    # HACK: only restrict imports on user's main module
    #
    # At this point we might actually just want to parse the file's AST
    # and match imports that way...
    is_main_mod = (globals is not None) and globals["__file__"] == "__fw_special__.py"
    first_atom = name.split(".")[0]
    if (not is_main_mod) or first_atom in ALLOWED_EXTERNAL_DEPS:
        assert orig_import is not None
        return orig_import(name, globals, locals, fromlist, level)
    else:
        raise ImportError(f"Import of module {name} is not allowed")


def exec_flumina_script(
    file_path: os.PathLike, exec_mode=_ExecMode.EXPORT, device="cuda"
):
    if not os.path.isfile(file_path):
        raise RuntimeError(f"File {file_path} does not exist.")
    with open(file_path, "r") as file:
        code = file.read()
    orig_import = builtins.__import__
    builtins.__import__ = partial(restricted_import, orig_import=orig_import)
    try:
        with _with_exec_mode(exec_mode), torch.device(device):
            code_obj = compile(code, file_path, "exec")
            restricted_globals = {
                "__builtins__": builtins,
                "__name__": "__flumina_main__",
                "__import__": restricted_import,
                "__file__": "__fw_special__.py",
            }
            exec(code_obj, restricted_globals)
    finally:
        builtins.__import__ = orig_import

    if exec_mode == _ExecMode.EXPORT:
        return _get_exported_module(), code
    elif exec_mode == _ExecMode.SERVE:
        return _get_initialized_module(), code
    else:
        raise NotImplementedError("Unsupported exec_mode", exec_mode)



from typing import List, Optional
from pydantic import BaseModel, Field, validator


class Error(BaseModel):
    object: str = "error"
    type: str = "invalid_request_error"
    message: str


class StructuredError(BaseModel):
    name: str = "bad_request"
    errors: List[str] = []


class ErrorResponse(BaseModel):
    error: Error = Field(default_factory=Error)
    structured_error: StructuredError = StructuredError(default_factory=StructuredError)


class TextPrompt(BaseModel):
    text: str
    weight: Optional[int] = 1


StyleLiteral = str


class StylePresetParams(BaseModel):
    style_preset: Optional[StyleLiteral] = None


# Subset of TextToImageRequestBody specifically for Stability APIs
#
# return_latents is moved to a request header
class StabilityAPITextToImageRequestBody(BaseModel):
    model: str = "stable-diffusion-v3-0"
    prompt: str
    negative_prompt: Optional[str] = None
    aspect_ratio: Optional[str] = None
    cfg_scale: Optional[int] = None
    seed: int = 0

    # Safety
    apply_invisible_watermark: bool = False


class TextToImageRequestBody(StylePresetParams):
    model: str = "stable-diffusion-xl-1024-v1-0"
    cfg_scale: Optional[int] = None
    height: Optional[int] = None
    width: Optional[int] = None
    sampler: Optional[str] = None
    samples: int = 1
    steps: Optional[int] = None
    seed: int = 0
    prompt: Optional[str] = None
    negative_prompt: Optional[str] = None

    # Deprecated - use `prompt` and `negative_prompt`
    text_prompts: Optional[List[TextPrompt]] = None

    # Deprecated
    clip_guidance_preset: str = "NONE"

    # Safety
    safety_check: bool = False
    apply_invisible_watermark: bool = False

    # LoRA parameters
    lora_adapter_name: Optional[str] = None
    lora_weight_filename: Optional[str] = None

    return_latents: bool = False


# Subset of ImageToImageRequestBody specifically for Stability APIs
#
# return_latents is moved to a request header
class StabilityAPIImageToImageRequestBody(BaseModel):
    model: str = "stable-diffusion-v3-0"
    # init_image
    prompt: str
    init_image_mode: Optional[str] = None
    negative_prompt: Optional[str] = None
    aspect_ratio: Optional[str] = None
    cfg_scale: Optional[int] = None
    seed: int = 0
    crop_padding: bool = True

    # init_image_mode=IMAGE_STRENGTH
    image_strength: Optional[float] = None

    # init_image_mode=STEP_SCHEDULE
    step_schedule_start: Optional[float] = None
    step_schedule_end: Optional[float] = None

    # Safety
    apply_invisible_watermark: bool = False


class ImageToImageRequestBody(StylePresetParams):
    model: str = "stable-diffusion-xl-1024-v1-0"
    cfg_scale: Optional[int] = None
    sampler: Optional[str] = None
    samples: int = 1
    steps: Optional[int] = None
    seed: int = 0
    height: Optional[int] = None
    width: Optional[int] = None

    init_image_mode: str = "IMAGE_STRENGTH"
    image_strength: Optional[float]
    step_schedule_start: Optional[float] = 0.65
    step_schedule_end: Optional[float]
    crop_padding: bool = True

    # Safety
    safety_check: bool = False
    apply_invisible_watermark: bool = False

    return_latents: bool = False

    prompt: str = Field(default="")
    negative_prompt: Optional[str] = None

    # LoRA parameters
    lora_adapter_name: Optional[str] = None
    lora_weight_filename: Optional[str] = None

    @validator("image_strength", pre=True, always=True)
    def validate_image_strength(cls, v, values):
        if values["init_image_mode"] == "IMAGE_STRENGTH" and v is None:
            raise ValueError(
                "image_strength is required when init_image_mode is IMAGE_STRENGTH"
            )
        return v

    @validator("step_schedule_start", "step_schedule_end", pre=True, always=True)
    def validate_step_schedule(cls, v, values):
        if values["init_image_mode"] == "STEP_SCHEDULE" and v is None:
            raise ValueError(
                "Both step_schedule_start and step_schedule_end are required when init_image_mode is STEP_SCHEDULE"
            )
        return v

    # Deprecated
    clip_guidance_preset: str = "NONE"


class ImageToVideoRequestBody(BaseModel):
    model: str = "stable-video-diffusion-img2vid"

    height: int = 576
    width: int = 1024
    sampler: Optional[str] = None
    frames: Optional[int] = None
    steps: int = 25
    min_guidance_scale: float = 1.0
    max_guidance_scale: float = 3.0
    fps: int = 7
    motion_bucket_id: int = 127
    noise_aug_strength: float = 0.02
    decode_chunk_size: Optional[int] = None
    seed: int = 0
    safety_check: bool = False
    frame_interpolation_factor: Optional[int] = None
    output_video_bitrate: Optional[int] = None
    return_latents: bool = False

    infer_if_input_is_flagged_DEBUG_ONLY: bool = False


class AnimateAnyoneRequestBody(BaseModel):
    model: str = ""

    width: int = 512
    height: int = 512
    steps: int = 10
    audio_guidance_scale: float = 3.0
    context_guidance_scale: float = 0.0
    guidance_rescale: float = 0.7
    seed: int = 0
    output_video_bitrate: int = 8_388_608


class ControlNetRequestBody(StylePresetParams):
    model: str = "stable-diffusion-xl-1024-v1-0"
    cfg_scale: Optional[int] = None
    sampler: Optional[str] = None
    samples: int = 1
    steps: Optional[int] = None
    seed: int = 0
    height: Optional[int] = None
    width: Optional[int] = None

    crop_padding: bool = True

    control_net_name: str
    conditioning_scale: float = 0.5

    step_schedule_start: Optional[float] = 0.0
    step_schedule_end: Optional[float] = 1.0

    # Safety
    apply_invisible_watermark: bool = False
    safety_check: bool = False

    return_latents: bool = False

    prompt: str = Field(default="")
    negative_prompt: Optional[str] = None

    # LoRA parameters
    lora_adapter_name: Optional[str] = None
    lora_weight_filename: Optional[str] = None

    # Deprecated
    clip_guidance_preset: str = "NONE"


class QRCodeRequest(BaseModel):
    prompt: str
    height: int = 1024
    width: int = 1024



import unittest
import fireworks.client
from fireworks.client import api
from pydantic import BaseModel


class TestBasic(unittest.TestCase):
    def test_import(self):
        self.assertRegex(fireworks.client.__version__, r"^\d+")

    def test_backward_compatiblity(self):
        # doesn't throw in the client
        api.UsageInfo(prompt_tokens=2, total_tokens=3, foo=1)

        # make sure all structs allow extra fields
        checked = 0
        for name in dir(api):
            if name.startswith("_"):
                continue
            cls = getattr(api, name)
            if (
                not hasattr(cls, "__module__")
                or cls.__module__ != "fireworks.client.api"
            ):
                continue
            if not issubclass(cls, BaseModel):
                continue
            checked += 1
            # can be deleted when we move to Pydantic 2 everywhere
            if hasattr(cls, "__config__"):
                self.assertIn(cls.Config.extra, ["ignore", None])
            else:
                self.assertIn(cls.model_config.get("extra"), ["ignore", None])
        self.assertGreater(checked, 5)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gateway/status.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='gateway/status.proto',
  package='gateway',
  syntax='proto3',
  serialized_options=b'Z\n./;gateway',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x14gateway/status.proto\x12\x07gateway\"6\n\x06Status\x12\x1b\n\x04\x63ode\x18\x01 \x01(\x0e\x32\r.gateway.Code\x12\x0f\n\x07message\x18\x02 \x01(\t*\xb7\x02\n\x04\x43ode\x12\x06\n\x02OK\x10\x00\x12\r\n\tCANCELLED\x10\x01\x12\x0b\n\x07UNKNOWN\x10\x02\x12\x14\n\x10INVALID_ARGUMENT\x10\x03\x12\x15\n\x11\x44\x45\x41\x44LINE_EXCEEDED\x10\x04\x12\r\n\tNOT_FOUND\x10\x05\x12\x12\n\x0e\x41LREADY_EXISTS\x10\x06\x12\x15\n\x11PERMISSION_DENIED\x10\x07\x12\x13\n\x0fUNAUTHENTICATED\x10\x10\x12\x16\n\x12RESOURCE_EXHAUSTED\x10\x08\x12\x17\n\x13\x46\x41ILED_PRECONDITION\x10\t\x12\x0b\n\x07\x41\x42ORTED\x10\n\x12\x10\n\x0cOUT_OF_RANGE\x10\x0b\x12\x11\n\rUNIMPLEMENTED\x10\x0c\x12\x0c\n\x08INTERNAL\x10\r\x12\x0f\n\x0bUNAVAILABLE\x10\x0e\x12\r\n\tDATA_LOSS\x10\x0f\x42\x0cZ\n./;gatewayb\x06proto3'
)

_CODE = _descriptor.EnumDescriptor(
  name='Code',
  full_name='gateway.Code',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OK', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CANCELLED', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INVALID_ARGUMENT', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DEADLINE_EXCEEDED', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NOT_FOUND', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ALREADY_EXISTS', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PERMISSION_DENIED', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNAUTHENTICATED', index=8, number=16,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RESOURCE_EXHAUSTED', index=9, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FAILED_PRECONDITION', index=10, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ABORTED', index=11, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='OUT_OF_RANGE', index=12, number=11,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNIMPLEMENTED', index=13, number=12,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INTERNAL', index=14, number=13,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNAVAILABLE', index=15, number=14,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DATA_LOSS', index=16, number=15,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=90,
  serialized_end=401,
)
_sym_db.RegisterEnumDescriptor(_CODE)

Code = enum_type_wrapper.EnumTypeWrapper(_CODE)
OK = 0
CANCELLED = 1
UNKNOWN = 2
INVALID_ARGUMENT = 3
DEADLINE_EXCEEDED = 4
NOT_FOUND = 5
ALREADY_EXISTS = 6
PERMISSION_DENIED = 7
UNAUTHENTICATED = 16
RESOURCE_EXHAUSTED = 8
FAILED_PRECONDITION = 9
ABORTED = 10
OUT_OF_RANGE = 11
UNIMPLEMENTED = 12
INTERNAL = 13
UNAVAILABLE = 14
DATA_LOSS = 15



_STATUS = _descriptor.Descriptor(
  name='Status',
  full_name='gateway.Status',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='gateway.Status.code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='message', full_name='gateway.Status.message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=33,
  serialized_end=87,
)

_STATUS.fields_by_name['code'].enum_type = _CODE
DESCRIPTOR.message_types_by_name['Status'] = _STATUS
DESCRIPTOR.enum_types_by_name['Code'] = _CODE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Status = _reflection.GeneratedProtocolMessageType('Status', (_message.Message,), {
  'DESCRIPTOR' : _STATUS,
  '__module__' : 'gateway.status_pb2'
  # @@protoc_insertion_point(class_scope:gateway.Status)
  })
_sym_db.RegisterMessage(Status)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
